;; Apple IIe Robot - Copyright 2019 by Michael Kohn
;; Email: mike@mikekohn.net
;;   Web: http://www.mikekohn.net/
;;
;; Communicate with an Apple IIe and drive motors to move it around.
;; Add a software serial port for communication with a possible EMIC
;; voice synthesis module.

.avr8
.include "m328def.inc"

;; Default fuses are 1MHz
;; avrdude: safemode: hfuse reads as D9
;; avrdude: safemode: efuse reads as FF
;; avrdude: safemode: Fuses OK (E:FF, H:D9, L:62) - 0010
;;
;; Need 20MHz  (full swing crystal oscillator: 0111  SUT: 11)
;; avrdude: safemode: Fuses OK (E:FF, H:D9, L:E7)

; r0  = 0
; r1  = 1
; r2  =
; r3  =
; r4  =
; r5  = Divide by 2 in interrupt for r25
; r6  =
; r7  =
; r16 =
; r17 = temp
; r18 = temp
; r19 = temp
; r20 = UART received data
; r21 = temp in main
; r23 = bit count in IR / Joystick communication
; r24 = function param 0
; r25 = Servo 20ms counter
; r26 = Servo value
; r30 =
; r31 =

; Command byte:
; 7 6 5 4 3 2 1 0
; C C C C V V V V
;
; C = 0: set motor 0 value
; C = 1: set motor 1 value

IR_DATA equ SRAM_START
YAW equ IR_DATA + 0
PITCH equ IR_DATA + 1
THROTTLE equ IR_DATA + 2
YAW_CORRECTION equ IR_DATA + 3
INTERRUPT_COUNT equ 1000
SERVO_RESTING equ 18
SERVO_STRIKING equ 13

PIN_ENC0 equ 5
PIN_ENC1 equ 6
PIN_ENC2 equ 7
PIN_ENCB equ 0

.org 0x000
  rjmp start
.org 0x02c
  rjmp timer1_match_a_interrupt

start:
  ;; Disable interrupts.
  cli

  ;; Setup some registers
  eor r0, r0   ; r0 = constant 0
  eor r1, r1   ; r1 = constant 1
  inc r1

  ;; Set up PORTB, PORTC, PORTD
  ;; PB1 IR
  ;; PB2 Servo
  ;; PC0 motor 0 INA, /INB
  ;; PC1 motor 1 INA, /INB
  ;; PC2 motor 0 PWM
  ;; PC3 motor 1 PWM
  ;; PC4 DATA  (Apple IIe joystick port)
  ;; PC5 CLOCK (Apple IIe joystick port)
  ;; PD0 RXD Apple IIe
  ;; PD1 TXD Apple IIe
  ;; PD2 RXD soft UART (EMIC)
  ;; PD3 TXD soft UART (EMIC)
  ;; PD4 LED
  ;; PD5 encoder 0
  ;; PD6 encoder 0
  ;; PD7 encoder 1
  ;; PB0 encoder 1
  ldi r17, 0x04
  out DDRB, r17             ; PB0-PB1 are input, PB2 is output
  out PORTB, r0             ; turn off all of port B
  ldi r17, 0x0f
  out DDRC, r17             ; PC0-PC3 are output, PC4-PC5 are input
  out PORTC, r0             ; turn off all of port C
  ldi r17, 0x18 
  out DDRD, r17             ; PD2,PD5-PD7 is input, PD3-PD4 is output
  out PORTD, r0             ; turn off all of port D

  ;; Setup stack ptr
  ldi r17, RAMEND >> 8
  out SPH, r17
  ldi r17, RAMEND & 255
  out SPL, r17

  ;; Setup both UARTs
  rcall uart_hw_init
  rcall uart_sw_init

  ;; Setup TIMER1
  lds r17, PRR
  andi r17, 255 ^ (1 << PRTIM1)
  sts PRR, r17                    ; turn of power management bit on TIMER1

  ldi r17, (INTERRUPT_COUNT >> 8)
  sts OCR1AH, r17
  ldi r17, (INTERRUPT_COUNT & 0xff) ; compare to 1000 clocks (0.05ms)
  sts OCR1AL, r17

  ldi r17, (1 << OCIE1A)
  sts TIMSK1, r17                 ; enable interrupt comare A 
  sts TCCR1C, r0
  sts TCCR1A, r0                  ; normal counting (0xffff is top, count up)
  ldi r17, (1 << CS10) | ( 1 << WGM12)   ; CTC OCR1A
  sts TCCR1B, r17                 ; prescale = 1 from clock source

  ;; Set servo to inital position.
  ldi r26, SERVO_RESTING

  ;; Enable interrupts.
  sei

  rcall delay_1sec
  rcall delay_1sec
  rcall delay_1sec
  rcall delay_1sec
  rcall delay_1sec

  ldi r30, (speech_init * 2) & 0xff
  ldi r31, (speech_init * 2) >> 8
  rcall uart_sw_send_string

main:
  ;rjmp remote_control_mode
  ;jrcall blink_led
  ;rjmp main

  rcall read_joystick_port

  cpi r21, 1
  breq call_forward
  cpi r21, 2
  breq call_rotate
  cpi r21, 3
  breq set_servo_resting
  cpi r21, 4
  breq set_servo_striking
  cpi r21, 5
  breq say_1
  cpi r21, 6
  breq say_2
  cpi r21, 7
  breq say_3

  ldi r30, (speech_init * 2) & 0xff
  ldi r31, (speech_init * 2) >> 8
  rcall uart_sw_send_string

  rjmp main

call_rotate:
  rcall rotate
  rjmp main

call_forward:
  rcall forward
  rjmp main

say_1:
  ldi r30, (speech_1 * 2) & 0xff
  ldi r31, (speech_1 * 2) >> 8
  rcall uart_sw_send_string
  rjmp main

say_2:
  ldi r30, (speech_2 * 2) & 0xff
  ldi r31, (speech_2 * 2) >> 8
  rcall uart_sw_send_string
  rjmp main

say_3:
  ldi r30, (speech_3 * 2) & 0xff
  ldi r31, (speech_3 * 2) >> 8
  rcall uart_sw_send_string
  rjmp main

set_servo_resting:
  ldi r26, SERVO_RESTING
  rjmp main

set_servo_striking:
  ldi r26, SERVO_STRIKING
  rjmp main

delay_1sec:
  clr r17
delay_1sec_78:
  clr r20
delay_1sec_wait:
  cpi r20, 255
  brne delay_1sec_wait
  inc r17
  cpi r17, 78
  brne delay_1sec_78
  ret

blink_led:
  sbi PORTD, 4
  clr r17
blink_led_loop_1:
  rcall pause
  inc r17
  cpi r17, 4
  brne blink_led_loop_1
  cbi PORTD, 4
  clr r17
blink_led_loop_2:
  rcall pause
  inc r17
  cpi r17, 4
  brne blink_led_loop_2
  ret

pause:
  clr r20
pause_loop:
  cpi r20, 200
  brne pause_loop
  ret

.include "uart_hw.asm"
.include "uart_sw.asm"
;.include "ir_comm.asm"
.include "read_joystick_port.asm"
;.include "remote_control_mode.asm"
.include "rotate.asm"
.include "forward.asm"

timer1_match_a_interrupt:
  ; Save status register.
  in r7, SREG

  inc r20
  brne not_zero
  inc r28
not_zero:

  ; Increment r25 every other interrupt.
  ; 400 interrupts (200 counts on r25) is 20ms
  inc r5
  sbrs r5, 0
  inc r25
  cpi r25, 200
  brne not_20ms
  sbi PORTB, PB2
  ldi r25, 0
not_20ms:
  cp r25, r26
  brne exit_interrupt
  cbi PORTB, PB2

exit_interrupt:
  ; Restore status register.
  out SREG, r7
  reti

.align_bytes 2
speech_init:
  asciiz "\nP0\nV18\nSready for action\n"
.align_bytes 2
speech_1:
  asciiz "\nSanother visitor\n"
.align_bytes 2
speech_2:
  asciiz "\nSout of my way jackass\n"
.align_bytes 2
speech_3:
  asciiz "\nSyou\n"


